import { Injectable, Inject } from '@angular/core';
import { Router } from '@angular/router';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import { of } from 'rxjs';
import { catchError, exhaustMap, map, tap, take } from 'rxjs/operators';

import {
  PiService,
  Login2Response,
  ResponseStatus
} from '@ucap-webmessenger/pi';
import { NativeService, UCAP_NATIVE_SERVICE } from '@ucap-webmessenger/native';
import {
  DialogService,
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap-webmessenger/ui';

import {
  login,
  loginSuccess,
  loginFailure,
  loginRedirect,
  logout,
  logoutConfirmation,
  logoutConfirmationDismiss
} from './actions';
import { LoginInfo } from '../../../types';
import { AppAuthenticationService } from '../../../services/authentication.service';

@Injectable()
export class Effects {
  login$ = createEffect(() =>
    this.actions$.pipe(
      ofType(login),
      map(action => action),
      exhaustMap((params: { loginInfo: LoginInfo; rememberMe: boolean }) =>
        this.piService
          .login2({
            loginId: params.loginInfo.loginId,
            loginPw: params.loginInfo.loginPw,
            companyCode: params.loginInfo.companyCode
          })
          .pipe(
            map((res: Login2Response) => {
              if (res.status === ResponseStatus.Fail) {
                return loginFailure({ error: 'Failed' });
              } else {
                return loginSuccess({
                  loginInfo: params.loginInfo,
                  rememberMe: params.rememberMe,
                  login2Response: res
                });
              }
            }),
            catchError(error => of(loginFailure({ error })))
          )
      )
    )
  );

  loginSuccess$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(loginSuccess),
        tap(params => {
          this.nativeService
            .checkForUpdates()
            .pipe(
              take(1),
              map((update: boolean) => {
                if (!update) {
                  this.appAuthenticationService.login(
                    params.loginInfo,
                    params.rememberMe
                  );
                  this.router.navigate(['/messenger']);
                }
              }),
              catchError(error => of(error))
            )
            .subscribe();
        })
      ),
    { dispatch: false }
  );

  loginRedirect$ = createEffect(
    () =>
      this.actions$.pipe(
        ofType(loginRedirect),
        tap(authed => {
          this.router.navigate(['/account/login']);
        })
      ),
    { dispatch: false }
  );

  logout$ = createEffect(() =>
    this.actions$.pipe(
      ofType(logout),
      map(action => action),
      map(() => {
        this.appAuthenticationService.logout();
        return loginRedirect();
      })
    )
  );

  logoutConfirmation$ = createEffect(() =>
    this.actions$.pipe(
      ofType(logoutConfirmation),
      exhaustMap(async () => {
        const result = await this.dialogService.open<
          ConfirmDialogComponent,
          ConfirmDialogData,
          ConfirmDialogResult
        >(ConfirmDialogComponent, {
          width: '220px',
          data: {
            title: 'Logout',
            message: 'Logout ?'
          }
        });

        return result.choice;
      }),
      map(result => (result ? logout() : logoutConfirmationDismiss()))
    )
  );

  constructor(
    private actions$: Actions,
    private router: Router,
    private piService: PiService,
    private appAuthenticationService: AppAuthenticationService,
    private dialogService: DialogService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}
}
